/*Simon Says Open - Now ready for servo code (where marked: @TODO), to use operating a latch;
 *           ===============================================================================
 * http://www.instructables.com/id/Improved-Simon-Says-Code/
 *
Originaly made by Robert Spann
Code trimmed and sound effects added by digimike
Ron Miller (aka RonM9) Jun 2015: Removed fixed pin assignment dependences and simplified code
RRM Oct 2015: allowed for faster button inputs
RRM Mar 2016  adapted Simon_Says_OPEN version; prepared for use as a door opener
RRM Aug 2021  Converted from using the 'Tone' object library to builtin 'tone' function.
              So now Simon_Says_OPEN_v2, and it complies with #include Servo.
              
Buttons are to be set on there designated pins without pull down resistors
and connected to ground rather then +5.
*/

#include <Servo.h>
#include "toneNotes.h"

const int OPEN_CNT = 4;

byte SPEAKERPIN = 14; // D14 is aka A0, speaker was on pin 13
int starttune[] = {NOTE_C4, NOTE_F4, NOTE_C4, NOTE_F4, NOTE_C4, NOTE_F4, NOTE_C4, NOTE_F4, NOTE_G4, NOTE_F4, NOTE_E4, NOTE_F4, NOTE_G4};
int duration2[] = {100, 200, 100, 200, 100, 400, 100, 100, 100, 100, 200, 100, 500};
int cheer[] = {NOTE_C4, NOTE_C4, NOTE_G4, NOTE_C5, NOTE_G4, NOTE_C5};
int btnNote[] = {NOTE_G3, NOTE_A3, NOTE_B3, NOTE_C4};
int duration[] = {100, 100, 100, 300, 100, 300};
//boolean button[] = {2, 3, 4, 5}; //The four button input pins
//boolean ledpin[] = {8, 9, 10, 11};  // LED pins
int button[] = {5, 6, 7, 8}; //The four button input pins
int ledpin[] = {2, 3, 17, 16};  // LED pins
int turn = 0;  // turn counter
int buttonstate = 0;  // button state checker
int randomArray[100]; //Intentionally long to store up to 100 inputs (doubtful anyone will get this far)
int inputArray[100];
bool wait4nextBtn = true;

// ===================================================
void setup()
{
  int note;
  unsigned long seed;
  
  Serial.begin(9600);
  Serial.println("Simon Says Open v2");
  pinMode(SPEAKERPIN,OUTPUT);
  noTone(SPEAKERPIN);

  // @TODO HERE DO SERVO SETUP
  // @TODO INITIALIZATION SERVO TO CLOSED (or NORMALLY DESIRED STATE)

  for (int x = 0; x < 4; x++) // LED pins are outputs
  {
    pinMode(ledpin[x], OUTPUT);
    pinMode(button[x], INPUT);  // button pins are inputs
    digitalWrite(button[x], HIGH);  // enable internal pullup; buttons start in high position; logic reversed
  }

  for (int thisNote = 0; thisNote < 13; thisNote ++) {
    // play the next note:
    note = starttune[thisNote];
    tone(SPEAKERPIN,note);
    if (note == NOTE_C4)      digitalWrite(ledpin[0], HIGH);
    else if (note == NOTE_F4) digitalWrite(ledpin[1], HIGH);
    else if (note == NOTE_G4) digitalWrite(ledpin[2], HIGH);
    else if (note == NOTE_E4) digitalWrite(ledpin[3], HIGH);

    // hold the note:
    delay(duration2[thisNote]);
    // stop for the next note:
    noTone(SPEAKERPIN);
    allLEDs(LOW);
    delay(25);
  }

  seed=42371949;  // what you set here will determine the code used
  Serial.print("Random Seed used: "); Serial.println(seed);
  randomSeed(seed); //Added to control the "random" #s used in the randomArray for the output function
  delay(1000);
}

// ========================================================
void loop()  // This 'loop' is automatically ran repeatedly
{
  for (int level = 0; level <= 99; level++)
  {
    allLEDs(LOW);
    
    Serial.print("\n Turn: ");   // Some serial output to follow along
    Serial.print(turn);

    if (turn==OPEN_CNT) {
      // @TODO HERE ADD CODE FOR SERVO OPEN (or desired special state)
      Serial.println("\n Simon Says Open the Lock ");
      playCheer();
      delay(5000);  // allow time for person to acually Open the door
      // @TODO HERE ADD CODE FOR SERVO RETURN TO NORMAL STATE
      Serial.println(" Re-Close ");
      wait4nextBtn = true;
    }
    
    if (wait4nextBtn) {
      wait4nextBtn = false;
      bool btnPushed = false;
      while (! btnPushed) { // wait for a key-press before starting all over
        for (int y = 0; y < 4; y++)
        { if (digitalRead(button[y]) == LOW) //Checking for button push
          btnPushed = true; 
        }
      }
      turn = -1; //Reset turn value so the game starts over
      exit;
    }
    
    // fill up the array to be matched by the player
    randomArray[turn] = random(1, 5); //Assigning a random number (1-4) to the randomArray[turn count]
    Serial.print("  Rand#: ");
    Serial.println(randomArray[turn]);

    for (int x = 0; x <= turn; x++)
    {
      Serial.print(randomArray[x]);
      byte btn = randomArray[x];  // logical button/led # (1-4)
      digitalWrite(ledpin[btn - 1], HIGH);
      tone(SPEAKERPIN,btnNote[btn - 1], 100);
      delay(400);
      digitalWrite(ledpin[btn - 1], LOW);
      delay(100);
    }
    input();
  }
}

// ===================================================
//            local functions

void input() { //Function for allowing user input and checking input against the generated array
  bool btnPushed;

  for (int x = 0; x <= turn; x++)
  { // for each one in the current sequence wait for a button to be pushed 
    // if it is the correct one we keep looping through the sequence

    btnPushed = false;
    while (! btnPushed) {
      for (int y = 0; y < 4; y++)
      {
        buttonstate = digitalRead(button[y]);
        byte btn = y + 1; // logical button # (1-4)

        if (buttonstate == LOW)  //Checking for button push
        {
          btnPushed = true;
          digitalWrite(ledpin[y], HIGH);
          tone(SPEAKERPIN, btnNote[btn - 1], 100);
          delay(100);  // insures minimum LED illumination
          inputArray[x] = btn;
          Serial.print("  btn pushed: ");
          Serial.print(btn);
// was a poor way to allow for button release:         delay(250);
          wait_BtnRelease();  // much better this way
          digitalWrite(ledpin[y], LOW);
          if (inputArray[x] != randomArray[x]) { //Checks value input by user and checks it against
            fail();                              //the value in the same spot on the generated array
            //The fail function is called if it does not match
            exit;
          }
        }
      }
    }  // end while
  }
  delay(500);
  turn++; //Increments the turn count, also the last action before starting the output function over again
}

// ------------------------------------------------------
void playCheer() { //Function used if the player fails to match the sequence
    allLEDs(HIGH);

    for (int thisNote = 0; thisNote < 6; thisNote ++) {
      // play the next note:
      tone(SPEAKERPIN, cheer[thisNote]);
      // hold the note:
      delay(duration[thisNote]);
      // stop for the next note:
      noTone(SPEAKERPIN);
      delay(25);
    }

    allLEDs(LOW);
    delay(1000);
}

void fail() { //Function used if the player fails to match the sequence

  for (int y = 0; y <= 2; y++)
  { //Flashes lights for failure
    allLEDs(HIGH);
    tone(SPEAKERPIN, NOTE_G3, 300);
    delay(200);
    allLEDs(LOW);
    tone(SPEAKERPIN, NOTE_C3, 300);
    delay(200);
  }
  delay(500);
  turn = -1; //Resets turn value so the game starts over without need for a reset button
}

// ------------------------------------------------------
void allLEDs(byte state) {
  digitalWrite(ledpin[0], state);
  digitalWrite(ledpin[1], state);
  digitalWrite(ledpin[2], state);
  digitalWrite(ledpin[3], state);
}

void wait_BtnRelease() {
  bool btnStillDown;
  int debounce=0; // need depends on button used. mine caused some double trips
    while (debounce<2) {
      delay(5);
      btnStillDown = false;
      for (int y = 0; y < 4; y++)
      {
        buttonstate = digitalRead(button[y]);
        if (buttonstate == LOW)  //Checking for button push
          btnStillDown = true;
      }
      if (btnStillDown) debounce=0;
      else debounce++;
    }
}
